"""Tests for OpenRouter LLM plugin."""

import os

import pytest
from dotenv import load_dotenv

from vision_agents.core.agents.conversation import Message, InMemoryConversation
from vision_agents.core.llm.events import (
    LLMResponseChunkEvent,
)
from vision_agents.plugins.openrouter import LLM

load_dotenv()


class TestOpenRouterLLM:
    """Test suite for OpenRouter LLM class."""

    def assert_response_successful(self, response):
        """Utility method to verify a response is successful.

        A successful response has:
        - response.text is set (not None and not empty)
        - response.exception is None

        Args:
            response: LLMResponseEvent to check
        """
        assert response.text is not None, "Response text should not be None"
        assert len(response.text) > 0, "Response text should not be empty"
        assert not hasattr(response, "exception") or response.exception is None, (
            f"Response should not have an exception, got: {getattr(response, 'exception', None)}"
        )

    def test_message(self):
        """Test basic message normalization."""
        messages = LLM._normalize_message("say hi")
        assert isinstance(messages[0], Message)
        message = messages[0]
        assert message.original is not None
        assert message.content == "say hi"

    def test_advanced_message(self):
        """Test advanced message format with image."""
        img_url = "https://upload.wikimedia.org/wikipedia/commons/thumb/d/d5/2023_06_08_Raccoon1.jpg/1599px-2023_06_08_Raccoon1.jpg"

        advanced = [
            {
                "role": "user",
                "content": [
                    {"type": "input_text", "text": "what do you see in this image?"},
                    {"type": "input_image", "image_url": f"{img_url}"},
                ],
            }
        ]
        messages = LLM._normalize_message(advanced)
        assert messages[0].original is not None

    @pytest.fixture
    async def llm(self) -> LLM:
        """Fixture for OpenRouter LLM with z-ai/glm-4.6 model."""
        if not os.environ.get("OPENROUTER_API_KEY"):
            pytest.skip("OPENROUTER_API_KEY environment variable not set")
        
        llm = LLM(model="anthropic/claude-haiku-4.5")
        llm._conversation = InMemoryConversation("be friendly", [])
        return llm

    @pytest.mark.integration
    async def test_simple(self, llm: LLM):
        """Test simple response generation."""
        response = await llm.simple_response(
            "Explain quantum computing in 1 paragraph",
        )

        self.assert_response_successful(response)

    @pytest.mark.integration
    async def test_native_api(self, llm: LLM):
        """Test native OpenAI-compatible API."""
        response = await llm.create_response(
            input="say hi", instructions="You are a helpful assistant."
        )

        self.assert_response_successful(response)
        assert hasattr(response.original, "id")  # OpenAI-compatible response has id

    @pytest.mark.integration
    async def test_streaming(self, llm: LLM):
        """Test streaming response."""
        streamingWorks = False

        @llm.events.subscribe
        async def passed(event: LLMResponseChunkEvent):
            nonlocal streamingWorks
            streamingWorks = True

        response = await llm.simple_response(
            "Explain quantum computing in 1 paragraph",
        )

        await llm.events.wait()

        self.assert_response_successful(response)
        assert streamingWorks, "Streaming should have generated chunk events"

    @pytest.mark.integration
    async def test_memory(self, llm: LLM):
        """Test conversation memory using simple_response."""
        await llm.simple_response(
            text="There are 2 dogs in the room",
        )
        response = await llm.simple_response(
            text="How many paws are there in the room?",
        )
        
        self.assert_response_successful(response)
        assert "8" in response.text or "eight" in response.text.lower(), (
            f"Expected '8' or 'eight' in response, got: {response.text}"
        )

    @pytest.mark.integration
    async def test_native_memory(self, llm: LLM):
        """Test conversation memory using native API."""
        await llm.create_response(
            input="There are 2 dogs in the room",
        )
        response = await llm.create_response(
            input="How many paws are there in the room?",
        )
        
        self.assert_response_successful(response)
        assert "8" in response.text or "eight" in response.text.lower(), (
            f"Expected '8' or 'eight' in response, got: {response.text}"
        )

    @pytest.mark.integration
    async def test_instruction_following(self):
        """Test that the LLM follows system instructions."""
        if not os.environ.get("OPENROUTER_API_KEY"):
            pytest.skip("OPENROUTER_API_KEY environment variable not set")

        pytest.skip("instruction following doesnt always work")
        llm = LLM(model="anthropic/claude-haiku-4.5")
        llm._set_instructions("Only reply in 2 letter country shortcuts")

        response = await llm.simple_response(
            text="Which country is rainy, protected from water with dikes and below sea level?",
        )

        self.assert_response_successful(response)
        assert "nl" in response.text.lower(), (
            f"Expected 'NL' in response, got: {response.text}"
        )




import asyncio
import logging
from uuid import uuid4

from dotenv import load_dotenv

from vision_agents.core import User
from vision_agents.core.agents import Agent
from vision_agents.plugins import openrouter, getstream, elevenlabs, deepgram, smart_turn

load_dotenv()

logging.basicConfig(level=logging.INFO, format="%(asctime)s %(levelname)s [call_id=%(call_id)s] %(name)s: %(message)s")
logger = logging.getLogger(__name__)


async def start_agent() -> None:
    """Example agent using OpenRouter LLM.
    
    This example demonstrates how to use the OpenRouter plugin with a Vision Agent.
    OpenRouter provides access to multiple LLM providers through a unified API.
    
    Set OPENROUTER_API_KEY environment variable before running.
    """
    agent = Agent(
        edge=getstream.Edge(),
        agent_user=User(name="OpenRouter AI"),
        instructions="Be helpful and friendly to the user",
        llm=openrouter.LLM(
            model="openai/gpt-4o",  # Can also use other models like anthropic/claude-3-opus
        ),
        tts=elevenlabs.TTS(),
        stt=deepgram.STT(),
        turn_detection=smart_turn.TurnDetection(
            buffer_in_seconds=2.0, confidence_threshold=0.5
        )
    )
    await agent.create_user()

    call = agent.edge.client.video.call("default", str(uuid4()))
    await agent.edge.open_demo(call)

    with await agent.join(call):
        await asyncio.sleep(5)
        await agent.llm.simple_response(text="Hello! I'm powered by OpenRouter.")
        await agent.finish()


if __name__ == "__main__":
    asyncio.run(start_agent())




"""OpenRouter LLM implementation using OpenAI-compatible API."""
import os
from typing import Any

from vision_agents.plugins.openai import LLM as OpenAILLM


class OpenRouterLLM(OpenAILLM):
    """OpenRouter LLM that extends OpenAI LLM with OpenRouter-specific configuration.

    It proxies the regular models by setting base url.
    It supports create response like the regular openAI API. It doesn't support conversation id, so that requires customization

    TODO:
    - Use manual conversation storage
    """

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str = "https://openrouter.ai/api/v1",
        model: str = "openrouter/andromeda-alpha",
        **kwargs: Any,
    ) -> None:
        """Initialize OpenRouter LLM.
        
        Args:
            api_key: OpenRouter API key. If not provided, uses OPENROUTER_API_KEY env var.
            base_url: OpenRouter API base URL.
            model: Model to use. Defaults to openai/gpt-4o.
            **kwargs: Additional arguments passed to OpenAI LLM.
        """
        if api_key is None:
            api_key = os.environ.get("OPENROUTER_API_KEY")
        super().__init__(
            api_key=api_key,
            base_url=base_url,
            model=model,
            **kwargs,
        )

    async def create_conversation(self):
        # Do nothing, dont call super
        pass

    def add_conversation_history(self, kwargs):
        # Use the manual storage
        # ensure the AI remembers the past conversation
        # TODO: there are additional formats to support here.
        new_messages = kwargs["input"]
        if not isinstance(new_messages, list):
            new_messages = [dict(content=new_messages, role="user", type="message")]
        if hasattr(self, '_conversation') and self._conversation:
            old_messages = [m.original for m in self._conversation.messages]
            kwargs["input"] = old_messages + new_messages
            # Add messages to conversation
            normalized_messages = self._normalize_message(new_messages)
            for msg in normalized_messages:
                self._conversation.messages.append(msg)

